package com.supermap.sample.extendREST;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import org.restlet.data.MediaType;
import com.supermap.services.components.commontypes.Point2D;
import com.supermap.services.rest.decoders.Decoder;

/**
 * 针对 distance 资源的扩展参数解析器类。
 *
 * JSON 格式的参数：point2Ds=[{x: 23.00,y:34.00},{x: 53.55,y:12.66},{x: 73.88,y:12.6}]&unit=METER
 * 预期可解析参数：point2Ds=[(x:23.00,y:34.00);(x:53.55,y:12.66);(x:73.88,y:12.6)]&unit=METER
 * <p>
 * Extend the decoder for the distance resource.
 *
 * JSON foramt parameter: point2Ds=[{x: 23.00,y:34.00},{x: 53.55,y:12.66},{x: 73.88,y:12.6}]&unit=METER
 * Expected parsible parameter: point2Ds=[(x:23.00,y:34.00);(x:53.55,y:12.66);(x:73.88,y:12.6)]&unit=METER
 * </p>
 */

public class MyDecoder extends Decoder{

	@Override
	protected List<MediaType> createSupportedMediaTypes() {
		// TODO Auto-generated method stub
        List<MediaType> mediaTypes = new ArrayList<MediaType>();
        mediaTypes.add(new MediaType("application/custom"));
        return mediaTypes;
	}

	@Override
	public List toList(String text, Class elementClass) throws Exception {
		List result = null;
		Object obj = this.toObject(text, Point2D.class);
		if(obj == null){
			return null;
		}
		Class eleClz = obj.getClass().getComponentType();
		if(eleClz .equals(Point2D.class)){
			List list = new ArrayList();
			Point2D[] points = (Point2D[])(obj);
			for(Point2D point: points){
				list.add(point);
			}
			result = list;
		}
		return result;
	}

	@Override
	public Map<String, Object> toMap(String str, Map<String, Class> nameClassMapping) {
		Map<String, Object> mapping = new HashMap() ;
		try{
		mapping = (Map<String, Object>)this.toObject(str, Point2D.class);
		}catch(Exception e){
			e.printStackTrace();
		}
		return mapping;
	}

	@Override
	public Object toObject(String mytypeStr, Class targetClass) throws Exception {
		if(targetClass == null){
			throw new NullPointerException("param targetClass can not be null");
		}
		if(mytypeStr == null){
			return null;
		}
		mytypeStr = mytypeStr.trim();
		Object resultObj = null;

		//转换数组。
		//mytypeStr以“[”开头，“]”结尾时，表示一个point2Ds
	    //Convert the string
        //If mytypeStr starts with "[" and ends with "]", it is a point2Ds
		if(mytypeStr.startsWith("[") && mytypeStr.endsWith("]")){
			mytypeStr = mytypeStr.substring(1);
			mytypeStr = mytypeStr.substring(0,mytypeStr.length() - 1);
			mytypeStr = mytypeStr.trim();
			if(mytypeStr .equals("")){
				return new Object[0];
			}else{
			     //There is only one Point2D object in the array
				if(mytypeStr.lastIndexOf("(") == 0){
					Point2D[] points = new Point2D[1];
					//
					Point2D point = (Point2D)this.toObject(mytypeStr, Point2D.class);
					points[0] = point;
					resultObj = points;
				}else{
					String[] objValueStrs = mytypeStr.split(";");
					Point2D[] points = new Point2D[objValueStrs.length];
					for(int i = 0 ; i < points.length ;i++){
						String objValueStr =  objValueStrs[i];
						Point2D point = (Point2D)this.toObject(objValueStr, Point2D.class);
						points[i] = point;
					}
					resultObj = points;
				}
			}
		}else if (Enum.class.isAssignableFrom(targetClass)) {
			//当 targetClass 是枚举类型时，按枚举类型解析 mytypeStr
			//这里targetClass是 Unit时，把 mytypeStr转化成一个Unit枚举对象
            //If targetClass is an enumeration, parse mytypeStr as an enumeration type
            //If targetClass is an Unit, converts mytypeStr into an Unit enumeration object
            resultObj = Enum.valueOf(targetClass, mytypeStr);
		}else{
		    Map<String ,String> nameValueMapping = this.getMap(mytypeStr);
	        if(nameValueMapping == null){
	            return null;
	        }
	        if(targetClass.equals(Point2D.class)){
	            if(nameValueMapping.size() == 0){
	                return new Point2D();
	            }else{
	                Point2D pp = new Point2D();
	                String strX = nameValueMapping.get("x");
	                String strY = nameValueMapping.get("y");
	                try{
	                    double x = Double.parseDouble(strX);
	                    double y = Double.parseDouble(strY);
	                    pp.x = x;
	                    pp.y = y;
	                    resultObj = pp;
	                }catch(NumberFormatException e){
	                    e.printStackTrace();
	                }
	            }
	        }
		}
        return resultObj;
	}


	@Override
	public Set toSet(String arg0, Class arg1) throws Exception {
		Set result = null;
		Object obj = this.toObject(arg0, Point2D.class);
		if(obj == null){
			return null;
		}
		Class eleClz = obj.getClass().getComponentType();
		if(eleClz .equals(Point2D.class)){
			Set set = new TreeSet();
			Point2D[] points = (Point2D[])(obj);
			for(Point2D point: points){
				set.add(point);
			}
			result = set;
		}
		return result;
	}

	private Map<String ,String> getMap(String str){
		Map<String, String> mapping = new HashMap<String, String>();
		if(str == null){
			return null;
		}
		str = str.trim();
		if(str.startsWith("(") && str.endsWith(")")){
			str = str.substring(1);
			str = str.substring(0,str.length() -1);
			str = str.trim();
			if(str .equals("")){
				return new HashMap();
			}else{
				String[] entryStr = str .split(",");
				for(int i=0;i<entryStr.length;i++){
					entryStr[i]=entryStr[i].trim();
					if(entryStr[i] .indexOf(':') == -1){
						continue;
					}
					String[] element=entryStr[i].split(":");
					if(element.length != 2){
						System.out.println("String" + entryStr + " contains multiple: ");
					}
					if(element[1]!=null){
						try {
							mapping.put(element[0].trim(),element[1].trim());
						} catch (Exception e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					}
				}
			}
		}else{
			throw new IllegalArgumentException("Invalid string format");
		}
	return mapping;
	}
}